import aiohttp


class Api:

    base_url = "https://www.devrant.io/api/"

    def __init__(self, username=None, password=None):
        self.username = username
        self.password = password
        self.auth = None
        self.app_id = 3
        self.user_id = None
        self.token_id = None
        self.token_Key = None
        self.session = None

    def patch_auth(self, request_dict=None):
        auth_dict = {"app": self.app_id}
        if self.auth:
            auth_dict.update(
                user_id=self.user_id, token_id=self.token_id, token_key=self.token_key
            )
        if not request_dict:
            return auth_dict
        request_dict.update(auth_dict)
        return request_dict

    def patch_url(self, url: str):
        return self.base_url.rstrip("/") + "/" + url.lstrip("/")

    async def login(self):
        if not self.username or not self.password:
            raise Exception("No authentication defails supplied.")
        async with self as session:
            response = await session.post(
                url=self.patch_url("users/auth-token"),
                data={
                    "username": self.username,
                    "password": self.password,
                    "app": self.app_id,
                },
            )
            obj = await response.json()
            if not obj.get("success"):
                return False
            self.auth = obj.get("auth_token")
            if not self.auth:
                return False
            self.user_id = self.auth.get("user_id")
            self.token_id = self.auth.get("id")
            self.token_key = self.auth.get("key")
        return self.auth and True or False

    async def ensure_login(self):
        if not self.auth:
            return await self.login()
        return True

    async def __aenter__(self):
        self.session = aiohttp.ClientSession()
        return self.session

    async def __aexit__(self, *args, **kwargs):
        await self.session.close()
        self.session = None

    async def post_comment(self, rant_id, comment):
        response = None
        if not await self.ensure_login():
            return False
        async with self as session:
            response = await session.post(
                url=self.patch_url(f"devrant/rants/{rant_id}/comments"),
                data=self.patch_auth({"comment": comment, "plat": 2}),
            )
        obj = await response.json()
        return obj.get("success", False)

    async def get_comment(self, id_):
        response = None
        async with self as session:
            response = await session.get(
                url=self.patch_url("comments/" + str(id_)), params=self.patch_auth()
            )
        obj = await response.json()

        print(obj)
        if not obj.get("success"):
            return None

        return obj.get("comment")

    async def get_profile(self, id_):
        response = None
        async with self as session:
            response = await session.get(
                url=self.patch_url(f"users/{id_}"), params=self.patch_auth()
            )
        obj = await response.json()
        if not obj.get("success"):
            return None
        return obj.get("profile")

    async def search(self, term):
        async with self as session:
            response = await session.get(
                url=self.patch_url("devrant/search"),
                params=self.patch_auth({"term": term}),
            )
        obj = await response.json()
        if not obj.get("success"):
            return
        for result in obj.get("results", []):
            yield result

    async def get_rant(self, id):
        response = None
        async with self as session:
            response = await session.get(
                self.patch_url(f"devrant/rants/{id}"),
                params=self.patch_auth(),
            )
        return await response.json()

    async def get_rants(self, sort="recent", limit=20, skip=0):
        response = None
        async with self as session:
            response = await session.get(
                url=self.patch_url("devrant/rants"),
                params=self.patch_auth({"sort": sort, "limit": limit, "skip": skip}),
            )
        obj = await response.json()
        if not obj.get("success"):
            return
        for rant in obj.get("rants", []):
            yield rant

    async def get_user_id(self, username):
        response = None
        async with self as session:
            response = await session.get(
                url=self.patch_url("get-user-id"),
                params=self.patch_auth({"username": username}),
            )
        obj = await response.json()
        if not obj.get("success"):
            return None
        return obj.get("user_id")

    @property
    async def mentions(self):
        async for notif in self.notifs:
            if notif["type"] == "comment_mention":
                yield notif

    @property
    async def notifs(self):
        response = None
        if not await self.ensure_login():
            return
        async with self as session:
            response = await session.get(
                url=self.patch_url("users/me/notif-feed"), params=self.patch_auth()
            )
        for item in (await response.json()).get("data", {}).get("items", []):
            yield item
